@@ -16,20 +16,19 @@ limitations under the License.
 
 import json
 
+from importlib_metadata import version as v
 from langtrace.trace_attributes import Event, LLMSpanAttributes
-from opentelemetry import baggage
+from opentelemetry import baggage, trace
 from opentelemetry.trace import SpanKind
 from opentelemetry.trace.status import Status, StatusCode
-
+from opentelemetry.trace.propagation import set_span_in_context
+from langtrace_python_sdk.constants import LANGTRACE_SDK_NAME
 from langtrace_python_sdk.constants.instrumentation.common import (
     LANGTRACE_ADDITIONAL_SPAN_ATTRIBUTES_KEY,
     SERVICE_PROVIDERS,
 )
 from langtrace_python_sdk.constants.instrumentation.openai import APIS
 from langtrace_python_sdk.utils.llm import calculate_prompt_tokens, estimate_tokens
-from importlib_metadata import version as v
-
-from langtrace_python_sdk.constants import LANGTRACE_SDK_NAME
 
 
 def images_generate(original_method, version, tracer):
@@ -65,7 +64,9 @@ def images_generate(original_method, version, tracer):
         attributes = LLMSpanAttributes(**span_attributes)
 
         with tracer.start_as_current_span(
-            APIS["IMAGES_GENERATION"]["METHOD"], kind=SpanKind.CLIENT
+            APIS["IMAGES_GENERATION"]["METHOD"],
+            kind=SpanKind.CLIENT,
+            context=set_span_in_context(trace.get_current_span()),
         ) as span:
             for field, value in attributes.model_dump(by_alias=True).items():
                 if value is not None:
@@ -142,7 +143,9 @@ def async_images_generate(original_method, version, tracer):
         attributes = LLMSpanAttributes(**span_attributes)
 
         with tracer.start_as_current_span(
-            APIS["IMAGES_GENERATION"]["METHOD"], kind=SpanKind.CLIENT
+            APIS["IMAGES_GENERATION"]["METHOD"],
+            kind=SpanKind.CLIENT,
+            context=set_span_in_context(trace.get_current_span()),
         ) as span:
             items = attributes.model_dump(by_alias=True).items()
             for field, value in items:
@@ -187,6 +190,195 @@ def async_images_generate(original_method, version, tracer):
     return traced_method
 
 
+def images_edit(original_method, version, tracer):
+    """
+    Wrap the `edit` method of the `Images` class to trace it.
+    """
+
+    def traced_method(wrapped, instance, args, kwargs):
+        base_url = (
+            str(instance._client._base_url)
+            if hasattr(instance, "_client") and hasattr(instance._client, "_base_url")
+            else ""
+        )
+        service_provider = SERVICE_PROVIDERS["OPENAI"]
+        extra_attributes = baggage.get_baggage(LANGTRACE_ADDITIONAL_SPAN_ATTRIBUTES_KEY)
+
+        span_attributes = {
+            "langtrace.sdk.name": "langtrace-python-sdk",
+            "langtrace.service.name": service_provider,
+            "langtrace.service.type": "llm",
+            "langtrace.service.version": version,
+            "langtrace.version": v(LANGTRACE_SDK_NAME),
+            "url.full": base_url,
+            "llm.api": APIS["IMAGES_EDIT"]["ENDPOINT"],
+            "llm.model": kwargs.get("model"),
+            "llm.response_format": kwargs.get("response_format"),
+            "llm.image.size": kwargs.get("size"),
+            "llm.prompts": json.dumps(
+                [
+                    {
+                        "role": kwargs.get("user", "user"),
+                        "content": kwargs.get("prompt", []),
+                    }
+                ]
+            ),
+            "llm.top_k": kwargs.get("n"),
+            **(extra_attributes if extra_attributes is not None else {}),
+        }
+
+        attributes = LLMSpanAttributes(**span_attributes)
+
+        with tracer.start_as_current_span(
+            APIS["IMAGES_EDIT"]["METHOD"],
+            kind=SpanKind.CLIENT,
+            context=set_span_in_context(trace.get_current_span()),
+        ) as span:
+            for field, value in attributes.model_dump(by_alias=True).items():
+                if value is not None:
+                    span.set_attribute(field, value)
+            try:
+                # Attempt to call the original method
+                result = wrapped(*args, **kwargs)
+
+                response = []
+                # Parse each image object
+                for each_data in result.data:
+                    response.append(
+                        {
+                            "role": "assistant",
+                            "content": {
+                                "url": each_data.url,
+                                "revised_prompt": each_data.revised_prompt,
+                                "base64": each_data.b64_json,
+                            },
+                        }
+                    )
+
+                span.add_event(
+                    name="response",
+                    attributes={"llm.responses": json.dumps(response)},
+                )
+
+                span.set_status(StatusCode.OK)
+                return result
+            except Exception as err:
+                # Record the exception in the span
+                span.record_exception(err)
+
+                # Set the span status to indicate an error
+                span.set_status(Status(StatusCode.ERROR, str(err)))
+
+                # Reraise the exception to ensure it's not swallowed
+                raise
+
+    return traced_method
+
+
+class StreamWrapper:
+    def __init__(self, stream, span, prompt_tokens, function_call=False, tool_calls=False):
+        self.stream = stream
+        self.span = span
+        self.prompt_tokens = prompt_tokens
+        self.function_call = function_call
+        self.tool_calls = tool_calls
+        self.result_content = []
+        self.completion_tokens = 0
+
+    def __enter__(self):
+        self.span.add_event(Event.STREAM_START.value)
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self.span.add_event(Event.STREAM_END.value)
+        self.span.set_attribute(
+            "llm.token.counts",
+            json.dumps(
+                {
+                    "input_tokens": self.prompt_tokens,
+                    "output_tokens": self.completion_tokens,
+                    "total_tokens": self.prompt_tokens + self.completion_tokens,
+                }
+            ),
+        )
+        self.span.set_attribute(
+            "llm.responses",
+            json.dumps(
+                [
+                    {
+                        "role": "assistant",
+                        "content": "".join(self.result_content),
+                    }
+                ]
+            ),
+        )
+        self.span.set_status(StatusCode.OK)
+        self.span.end()
+
+    def __iter__(self):
+        return self
+
+    def __next__(self):
+        try:
+            chunk = next(self.stream)
+            self.process_chunk(chunk)
+            return chunk
+        except StopIteration:
+            raise
+
+    def process_chunk(self, chunk):
+        if hasattr(chunk, "model") and chunk.model is not None:
+            self.span.set_attribute("llm.model", chunk.model)
+        if hasattr(chunk, "choices") and chunk.choices is not None:
+            content = []
+            if not self.function_call and not self.tool_calls:
+                for choice in chunk.choices:
+                    if choice.delta and choice.delta.content is not None:
+                        token_counts = estimate_tokens(choice.delta.content)
+                        self.completion_tokens += token_counts
+                        content = [choice.delta.content]
+            elif self.function_call:
+                for choice in chunk.choices:
+                    if (
+                        choice.delta
+                        and choice.delta.function_call is not None
+                        and choice.delta.function_call.arguments is not None
+                    ):
+                        token_counts = estimate_tokens(
+                            choice.delta.function_call.arguments
+                        )
+                        self.completion_tokens += token_counts
+                        content = [choice.delta.function_call.arguments]
+            elif self.tool_calls:
+                for choice in chunk.choices:
+                    if choice.delta and choice.delta.tool_calls is not None:
+                        toolcalls = choice.delta.tool_calls
+                        content = []
+                        for tool_call in toolcalls:
+                            if (
+                                tool_call
+                                and tool_call.function is not None
+                                and tool_call.function.arguments is not None
+                            ):
+                                token_counts = estimate_tokens(
+                                    tool_call.function.arguments
+                                )
+                                self.completion_tokens += token_counts
+                                content.append(tool_call.function.arguments)
+            self.span.add_event(
+                Event.STREAM_OUTPUT.value,
+                {
+                    "response": (
+                        "".join(content)
+                        if len(content) > 0 and content[0] is not None
+                        else ""
+                    )
+                },
+            )
+            if content:
+                self.result_content.append(content[0])
+
+
 def chat_completions_create(original_method, version, tracer):
     """Wrap the `create` method of the `ChatCompletion` class to trace it."""
 
@@ -267,7 +459,9 @@ def chat_completions_create(original_method, version, tracer):
         # with tracer.start_as_current_span(APIS["CHAT_COMPLETION"]["METHOD"],
         #                                   kind=SpanKind.CLIENT) as span:
         span = tracer.start_span(
-            APIS["CHAT_COMPLETION"]["METHOD"], kind=SpanKind.CLIENT
+            APIS["CHAT_COMPLETION"]["METHOD"],
+            kind=SpanKind.CLIENT,
+            context=set_span_in_context(trace.get_current_span()),
         )
         for field, value in attributes.model_dump(by_alias=True).items():
             if value is not None:
@@ -337,7 +531,7 @@ def chat_completions_create(original_method, version, tracer):
                             json.dumps(function), kwargs.get("model")
                         )
 
-                return handle_streaming_response(
+                return StreamWrapper(
                     result,
                     span,
                     prompt_tokens,
@@ -351,98 +545,6 @@ def chat_completions_create(original_method, version, tracer):
             span.end()
             raise
 
-    def handle_streaming_response(
-        result, span, prompt_tokens, function_call=False, tool_calls=False
-    ):
-        """Process and yield streaming response chunks."""
-        result_content = []
-        span.add_event(Event.STREAM_START.value)
-        completion_tokens = 0
-        try:
-            for chunk in result:
-                if hasattr(chunk, "model") and chunk.model is not None:
-                    span.set_attribute("llm.model", chunk.model)
-                if hasattr(chunk, "choices") and chunk.choices is not None:
-                    if not function_call and not tool_calls:
-                        for choice in chunk.choices:
-                            if choice.delta and choice.delta.content is not None:
-                                token_counts = estimate_tokens(choice.delta.content)
-                                completion_tokens += token_counts
-                                content = [choice.delta.content]
-                    elif function_call:
-                        for choice in chunk.choices:
-                            if (
-                                choice.delta
-                                and choice.delta.function_call is not None
-                                and choice.delta.function_call.arguments is not None
-                            ):
-                                token_counts = estimate_tokens(
-                                    choice.delta.function_call.arguments
-                                )
-                                completion_tokens += token_counts
-                                content = [choice.delta.function_call.arguments]
-                    elif tool_calls:
-                        for choice in chunk.choices:
-                            tool_call = ""
-                            if choice.delta and choice.delta.tool_calls is not None:
-                                toolcalls = choice.delta.tool_calls
-                                content = []
-                                for tool_call in toolcalls:
-                                    if (
-                                        tool_call
-                                        and tool_call.function is not None
-                                        and tool_call.function.arguments is not None
-                                    ):
-                                        token_counts = estimate_tokens(
-                                            tool_call.function.arguments
-                                        )
-                                        completion_tokens += token_counts
-                                        content = content + [
-                                            tool_call.function.arguments
-                                        ]
-                                    else:
-                                        content = content + []
-                else:
-                    content = []
-                span.add_event(
-                    Event.STREAM_OUTPUT.value,
-                    {
-                        "response": (
-                            "".join(content)
-                            if len(content) > 0 and content[0] is not None
-                            else ""
-                        )
-                    },
-                )
-                result_content.append(content[0] if len(content) > 0 else "")
-                yield chunk
-        finally:
-            # Finalize span after processing all chunks
-            span.add_event(Event.STREAM_END.value)
-            span.set_attribute(
-                "llm.token.counts",
-                json.dumps(
-                    {
-                        "input_tokens": prompt_tokens,
-                        "output_tokens": completion_tokens,
-                        "total_tokens": prompt_tokens + completion_tokens,
-                    }
-                ),
-            )
-            span.set_attribute(
-                "llm.responses",
-                json.dumps(
-                    [
-                        {
-                            "role": "assistant",
-                            "content": "".join(result_content),
-                        }
-                    ]
-                ),
-            )
-            span.set_status(StatusCode.OK)
-            span.end()
-
     # return the wrapped method
     return traced_method
 
@@ -597,7 +699,7 @@ def async_chat_completions_create(original_method, version, tracer):
                             json.dumps(function), kwargs.get("model")
                         )
 
-                return ahandle_streaming_response(
+                return StreamWrapper(
                     result,
                     span,
                     prompt_tokens,
@@ -611,99 +713,6 @@ def async_chat_completions_create(original_method, version, tracer):
             span.end()
             raise
 
-    async def ahandle_streaming_response(
-        result, span, prompt_tokens, function_call=False, tool_calls=False
-    ):
-        """Process and yield streaming response chunks."""
-        result_content = []
-        span.add_event(Event.STREAM_START.value)
-        completion_tokens = 0
-        try:
-            content = []
-            async for chunk in result:
-                if hasattr(chunk, "model") and chunk.model is not None:
-                    span.set_attribute("llm.model", chunk.model)
-                if hasattr(chunk, "choices") and chunk.choices is not None:
-                    if not function_call and not tool_calls:
-                        for choice in chunk.choices:
-                            if choice.delta and choice.delta.content is not None:
-                                token_counts = estimate_tokens(choice.delta.content)
-                                completion_tokens += token_counts
-                                content = [choice.delta.content]
-                    elif function_call:
-                        for choice in chunk.choices:
-                            if (
-                                choice.delta
-                                and choice.delta.function_call
-                                and choice.delta.function_call.arguments is not None
-                            ):
-                                token_counts = estimate_tokens(
-                                    choice.delta.function_call.arguments
-                                )
-                                completion_tokens += token_counts
-                                content = [choice.delta.function_call.arguments]
-                    elif tool_calls:
-                        for choice in chunk.choices:
-                            tool_call = ""
-                            if choice.delta and choice.delta.tool_calls is not None:
-                                toolcalls = choice.delta.tool_calls
-                                content = []
-                                for tool_call in toolcalls:
-                                    if (
-                                        tool_call
-                                        and tool_call.function is not None
-                                        and tool_call.function.arguments is not None
-                                    ):
-                                        token_counts = estimate_tokens(
-                                            tool_call.function.arguments
-                                        )
-                                        completion_tokens += token_counts
-                                        content = content + [
-                                            tool_call.function.arguments
-                                        ]
-                                    else:
-                                        content = content + []
-                else:
-                    content = []
-                span.add_event(
-                    Event.STREAM_OUTPUT.value,
-                    {
-                        "response": (
-                            "".join(content)
-                            if len(content) > 0 and content[0] is not None
-                            else ""
-                        )
-                    },
-                )
-                result_content.append(content[0] if len(content) > 0 else "")
-                yield chunk
-        finally:
-            # Finalize span after processing all chunks
-            span.add_event(Event.STREAM_END.value)
-            span.set_attribute(
-                "llm.token.counts",
-                json.dumps(
-                    {
-                        "input_tokens": prompt_tokens,
-                        "output_tokens": completion_tokens,
-                        "total_tokens": prompt_tokens + completion_tokens,
-                    }
-                ),
-            )
-            span.set_attribute(
-                "llm.responses",
-                json.dumps(
-                    [
-                        {
-                            "role": "assistant",
-                            "content": "".join(result_content),
-                        }
-                    ]
-                ),
-            )
-            span.set_status(StatusCode.OK)
-            span.end()
-
     # return the wrapped method
     return traced_method
 
@@ -751,7 +760,9 @@ def embeddings_create(original_method, version, tracer):
             attributes["llm.user"] = kwargs.get("user")
 
         with tracer.start_as_current_span(
-            APIS["EMBEDDINGS_CREATE"]["METHOD"], kind=SpanKind.CLIENT
+            APIS["EMBEDDINGS_CREATE"]["METHOD"],
+            kind=SpanKind.CLIENT,
+            context=set_span_in_context(trace.get_current_span()),
         ) as span:
 
             for field, value in attributes.model_dump(by_alias=True).items():
@@ -816,7 +827,9 @@ def async_embeddings_create(original_method, version, tracer):
             attributes["llm.user"] = kwargs.get("user")
 
         with tracer.start_as_current_span(
-            APIS["EMBEDDINGS_CREATE"]["METHOD"], kind=SpanKind.CLIENT
+            APIS["EMBEDDINGS_CREATE"]["METHOD"],
+            kind=SpanKind.CLIENT,
+            context=set_span_in_context(trace.get_current_span()),
         ) as span:
 
             async for field, value in attributes.model_dump(by_alias=True).items():
